﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace tinyIE {
	public partial class MainForm : Form {
		public MainForm() {
			InitializeComponent();
			mainBrowser.CanGoBackChanged += new EventHandler(webBrowser1_CanGoBackChanged);
			mainBrowser.CanGoForwardChanged += new EventHandler(webBrowser1_CanGoForwardChanged);

			topPanel.Height = addressTextBox.Height;
			backButton.Width = forwardButton.Width = homeButton.Width = topPanel.Height;

			if (Program.Arguments.Length != 0) {
				mainBrowser.Url = new Uri(Program.Arguments[0]);
			}
		}

		void webBrowser1_CanGoForwardChanged(object sender, EventArgs e) {
			forwardButton.Enabled = mainBrowser.CanGoForward;
		}

		void webBrowser1_CanGoBackChanged(object sender, EventArgs e) {
			backButton.Enabled = mainBrowser.CanGoBack;
		}

		private void webBrowser1_Navigated(object sender, WebBrowserNavigatedEventArgs e) {
			addressTextBox.Text = mainBrowser.Url.ToString();
			if (mainBrowser.Document == null || string.IsNullOrEmpty(mainBrowser.Document.Title)) {
				this.Text = "TinyIE";
			}
			else {
				this.Text = mainBrowser.Document.Title + " - TineIE";
			}
		}


		private void timer1_Tick(object sender, EventArgs e) {
			contextMenu.Show(Control.MousePosition);
		}

		private void quitToolStripMenuItem_Click(object sender, EventArgs e) {
			Application.Exit();
		}

		private void modalToolStripMenuItem_Click(object sender, EventArgs e) {
			this.TopMost = !this.TopMost;
			ToolStripMenuItem m = sender as ToolStripMenuItem;
			m.Checked = this.TopMost;
		}

		bool resizeDrag, moveDrag;
		Point dragPrev;
		private void panel2_MouseDown(object sender, MouseEventArgs e) {
			if (e.Button == MouseButtons.Left) {
				this.SuspendLayout();
				moveDrag = true;
			}
			dragPrev = e.Location;
		}

		private void panel3_MouseDown(object sender, MouseEventArgs e) {
			if (e.Button == MouseButtons.Left) {
				resizeDrag = true;
			}
			dragPrev = e.Location;
		}

		private void panel2_MouseMove(object sender, MouseEventArgs e) {
			if (resizeDrag) {
				// サイズ変更
				this.Width += (e.Location.X - dragPrev.X);
				this.Height += (e.Location.Y - dragPrev.Y);
	//			dragPrev = e.Location;
			}
			else if (moveDrag) {
				// 移動
				this.Left += (e.Location.X - dragPrev.X);
				this.Top += (e.Location.Y - dragPrev.Y);
			}
		}

		private void panel2_MouseUp(object sender, MouseEventArgs e) {
			if (e.Button == MouseButtons.Left)
				moveDrag = false;
			this.ResumeLayout();

		}

		private void panel3_MouseUp(object sender, MouseEventArgs e) {
			if (e.Button == MouseButtons.Left)
				resizeDrag = false;

		}

		private void panel2_MouseClick(object sender, MouseEventArgs e) {
			if (e.Button == MouseButtons.Right)
				contextMenu.Show(Control.MousePosition);
		}

		private void maximizeToolStripMenuItem_Click(object sender, EventArgs e) {
			ToolStripMenuItem m = sender as ToolStripMenuItem;
			if (this.WindowState == FormWindowState.Normal) {
				this.WindowState = FormWindowState.Maximized;
				mainPanel.BorderStyle = BorderStyle.None;
			}
			else {
				this.WindowState = FormWindowState.Normal;
				mainPanel.BorderStyle = BorderStyle.FixedSingle;
			}
		}

		private void contextMenuStrip1_Opening(object sender, CancelEventArgs e) {
			if (mainBrowser.Document != null&& mainBrowser.Document.Body != null) 
				zoomToolStripMenuItem.Enabled = true;
			else
				zoomToolStripMenuItem.Enabled = false;
			scrollBarToolStripMenuItem.Checked = mainBrowser.ScrollBarsEnabled;
		}

		private void toolStripMenuItem5_Click(object sender, EventArgs e) {
			changeZoom("1.0");
		}

		private void toolStripMenuItem6_Click(object sender, EventArgs e) {
			changeZoom("0.5");
		}

		private void changeZoom(string rate) {
			string currentStyle = mainBrowser.Document.Body.Style;
			if (currentStyle == null) currentStyle = "";
			string newStyle = null;
			int index = currentStyle.IndexOf("zoom", 0, 0, StringComparison.CurrentCultureIgnoreCase);
			if (index < 0) {
				newStyle = currentStyle + "zoom : " + rate + ";";
			}
			else {
				int index2 = currentStyle.IndexOf(';', index);
				if (index2 < -1) {
					newStyle = "";
				}
				else {
					newStyle = currentStyle.Substring(0, index + 1) + "zoom : " + rate + currentStyle.Substring(index2);
				}
			}
			mainBrowser.Document.Body.Style = newStyle;
		}

		private void toolStripMenuItem9_Click(object sender, EventArgs e) {
			changeZoom("1.5");
		}

		private void toolStripMenuItem10_Click(object sender, EventArgs e) {
			changeZoom("2.0");
		}

		private void toolStripMenuItem7_Click(object sender, EventArgs e) {
			changeZoom("0.25");
		}

		private void toolStripMenuItem8_Click(object sender, EventArgs e) {
			changeZoom("0.1");
		}

		private void handleToolStripMenuItem_Click(object sender, EventArgs e) {
			movePanel.Visible = resizePanel.Visible = !movePanel.Visible;

			if (!topPanel.Visible && !movePanel.Visible) {
				addressToolStripMenuItem_Click(addressToolStripMenuItem, e);
			}
		}

		private void addressToolStripMenuItem_Click(object sender, EventArgs e) {
			topPanel.Visible = !topPanel.Visible;
			if (!topPanel.Visible && !movePanel.Visible) {
				handleToolStripMenuItem_Click(handleToolStripMenuItem, e);
			}
		}

		private void copyURIToolStripMenuItem_Click(object sender, EventArgs e) {
			Clipboard.SetText(addressTextBox.Text);
			addressTextBox.SelectAll();
		}

		private void pasteURIToolStripMenuItem_Click(object sender, EventArgs e) {
			addressTextBox.Text = Clipboard.GetText();
			if (!addressTextBox.Visible) addressToolStripMenuItem_Click(this, e);
		}

		private void webBrowser1_BeforeNewWindow(object sender, WebBrowserExtendedNavigatingEventArgs e) {
			e.Cancel = true;
			Process.Start(Application.ExecutablePath, e.Url);
		}

		private void panel2_DoubleClick(object sender, EventArgs e) {
			Application.Exit();
		}

		private void textBox1_VisibleChanged(object sender, EventArgs e) {
			addressToolStripMenuItem.Checked = addressTextBox.Visible;

		}

		private void panel2_VisibleChanged(object sender, EventArgs e) {
			handleToolStripMenuItem.Checked = movePanel.Visible;
		}

		private void Form1_Resize(object sender, EventArgs e) {
			if (this.WindowState == FormWindowState.Maximized) {
				maximizeToolStripMenuItem.Checked = true;
			}
			else {
				maximizeToolStripMenuItem.Checked = false;
			}
		}

		private void textBox1_KeyPress(object sender, KeyPressEventArgs e) {
			if (e.KeyChar == '\r' || e.KeyChar == '\n') {
				mainBrowser.Navigate(addressTextBox.Text);
				e.Handled = true;
			}
		}

		private void webBrowser1_ProgressChanged(object sender, WebBrowserProgressChangedEventArgs e) {
			if (e.CurrentProgress == e.MaximumProgress) {
				Color c = Color.FromKnownColor(KnownColor.Highlight);
				movePanel.BackColor = c;
				resizePanel.BackColor = c;				
			}
		}

		private void webBrowser1_Navigating(object sender, WebBrowserNavigatingEventArgs e) {
			Color c = Color.FromArgb(Color.FromKnownColor(KnownColor.Highlight).ToArgb() ^ 0x00FFFFFF);
			movePanel.BackColor = c;
			resizePanel.BackColor = c;
		}

		private void toolStripMenuItem12_Click(object sender, EventArgs e) {
			mainBrowser.ScrollBarsEnabled = !mainBrowser.ScrollBarsEnabled;
		}

		private void button1_Click(object sender, EventArgs e) {
			mainBrowser.GoBack();
		}

		private void button2_Click(object sender, EventArgs e) {
			mainBrowser.GoForward();
		}

		private void button3_Click(object sender, EventArgs e) {
			mainBrowser.GoHome();
		}

	}


	/* from http://vsug.jp/tabid/63/forumid/45/threadid/9273/scope/posts/Default.aspx */
	#region
	public class WebBrowserExtendedNavigatingEventArgs : CancelEventArgs {
		private string _Url;
		public string Url {
			get { return _Url; }
		}

		private string _Frame;
		public string Frame {
			get { return _Frame; }
		}

		public WebBrowserExtendedNavigatingEventArgs(string url, string frame)
			: base() {
			_Url = url;
			_Frame = frame;
		}
	}

	public class ExtendedWebBrowser : System.Windows.Forms.WebBrowser {
		System.Windows.Forms.AxHost.ConnectionPointCookie cookie;
		WebBrowserExtendedEvents events;

		//This method will be called to give you a chance to create your own event sink
		protected override void CreateSink() {
			//MAKE SURE TO CALL THE BASE or the normal events won't fire
			base.CreateSink();
			events = new WebBrowserExtendedEvents(this);
			cookie = new System.Windows.Forms.AxHost.ConnectionPointCookie(this.ActiveXInstance, events, typeof(DWebBrowserEvents2));
		}

		protected override void DetachSink() {
			if (null != cookie) {
				cookie.Disconnect();
				cookie = null;
			}
			base.DetachSink();
		}

		//This new event will fire when the page is navigating
		public event EventHandler<WebBrowserExtendedNavigatingEventArgs> BeforeNavigate;
		public event EventHandler<WebBrowserExtendedNavigatingEventArgs> BeforeNewWindow;

		protected void OnBeforeNewWindow(string url, out bool cancel) {
			EventHandler<WebBrowserExtendedNavigatingEventArgs> h = BeforeNewWindow;
			WebBrowserExtendedNavigatingEventArgs args = new WebBrowserExtendedNavigatingEventArgs(url, null);
			if (null != h) {
				h(this, args);
			}
			cancel = args.Cancel;
		}

		protected void OnBeforeNavigate(string url, string frame, out bool cancel) {
			EventHandler<WebBrowserExtendedNavigatingEventArgs> h = BeforeNavigate;
			WebBrowserExtendedNavigatingEventArgs args = new WebBrowserExtendedNavigatingEventArgs(url, frame);
			if (null != h) {
				h(this, args);
			}
			//Pass the cancellation chosen back out to the events
			cancel = args.Cancel;
		}

		//This class will capture events from the WebBrowser
		class WebBrowserExtendedEvents : System.Runtime.InteropServices.StandardOleMarshalObject, DWebBrowserEvents2 {
			ExtendedWebBrowser _Browser;
			public WebBrowserExtendedEvents(ExtendedWebBrowser browser) { _Browser = browser; }

			//Implement whichever events you wish
			public void BeforeNavigate2(object pDisp, ref object URL, ref object flags, ref object targetFrameName, ref object postData, ref object headers, ref bool cancel) {
				_Browser.OnBeforeNavigate((string)URL, (string)targetFrameName, out cancel);
			}

			public void NewWindow3(object pDisp, ref bool cancel, ref object flags, ref object URLContext, ref object URL) {
				_Browser.OnBeforeNewWindow((string)URL, out cancel);
			}

		}

		[System.Runtime.InteropServices.ComImport(), System.Runtime.InteropServices.Guid("34A715A0-6587-11D0-924A-0020AFC7AC4D"),
		System.Runtime.InteropServices.InterfaceTypeAttribute(System.Runtime.InteropServices.ComInterfaceType.InterfaceIsIDispatch),
		System.Runtime.InteropServices.TypeLibType(System.Runtime.InteropServices.TypeLibTypeFlags.FHidden)]
		public interface DWebBrowserEvents2 {

			[System.Runtime.InteropServices.DispId(250)]
			void BeforeNavigate2(
				[System.Runtime.InteropServices.In,
				System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.IDispatch)] object pDisp,
				[System.Runtime.InteropServices.In] ref object URL,
				[System.Runtime.InteropServices.In] ref object flags,
				[System.Runtime.InteropServices.In] ref object targetFrameName, [System.Runtime.InteropServices.In] ref object postData,
				[System.Runtime.InteropServices.In] ref object headers,
				[System.Runtime.InteropServices.In,
				System.Runtime.InteropServices.Out] ref bool cancel);
			[System.Runtime.InteropServices.DispId(273)]
			void NewWindow3(
				[System.Runtime.InteropServices.In,
				System.Runtime.InteropServices.MarshalAs(System.Runtime.InteropServices.UnmanagedType.IDispatch)] object pDisp,
				[System.Runtime.InteropServices.In, System.Runtime.InteropServices.Out] ref bool cancel,
				[System.Runtime.InteropServices.In] ref object flags,
				[System.Runtime.InteropServices.In] ref object URLContext,
				[System.Runtime.InteropServices.In] ref object URL);
		}
	}
	#endregion

}
